import java.io.*;
import java.util.*;

public class Kruskal {
	private final int MAX_NODES = 300;
	private HashSet nodes[]; // Array of connected components
	private TreeSet allEdges; // Priority queue of Edge objects
	private Vector allNewEdges; // Edges in Minimal-Spanning Tree
	private long[][] routes;

	public static void main(String args[]) throws FileNotFoundException {
//		System.out.println("Running [Kruskal] - 2000");
		
		System.setOut(new PrintStream(new File("graph.out")));;
		Kruskal k = new Kruskal();
		k.readInGraphData("graph.in");
		k.performKruskal();
		k.printFinalEdges();
	}

	Kruskal() {
		// Constructor
		nodes = new HashSet[MAX_NODES]; // Create array for components
		allEdges = new TreeSet(new Edge()); // Create empty priority queue
		allNewEdges = new Vector(MAX_NODES); // Create vector for MST edges
		routes = new long[MAX_NODES][MAX_NODES]; // all routes
	}

	private void readInGraphData(String fileName) {
		try {
			FileReader file = new FileReader(fileName);
			BufferedReader buff = new BufferedReader(file);
			String line = buff.readLine();
			
			StringTokenizer firstLine = new StringTokenizer(line, " ");
			int N = Integer.valueOf(firstLine.nextToken()).intValue();
			int M = Integer.valueOf(firstLine.nextToken()).intValue();
		
			int k = 0; 
			line = buff.readLine();
			HashSet tempEdges = new HashSet();
			while (++k < M) {
				StringTokenizer tok = new StringTokenizer(line, " ");
				int from = Integer.valueOf(tok.nextToken()).intValue();
				int to = Integer.valueOf(tok.nextToken()).intValue();
				long cost = Long.valueOf(tok.nextToken()).longValue();

				tempEdges.add(new Edge(from, to, cost, k)); 
				if (nodes[from] == null) {
					// Create set of connect components [singleton] for this
					// node
					nodes[from] = new HashSet(2 * MAX_NODES);
					nodes[from].add(new Integer(from));
				}

				if (nodes[to] == null) {
					// Create set of connect components [singleton] for this
					// node
					nodes[to] = new HashSet(2 * MAX_NODES);
					nodes[to].add(new Integer(to));
				}

				line = buff.readLine();
			}
			
			for(int i=0; i<N;i++){
				int j=-1;
				line = buff.readLine();
				StringTokenizer tok = new StringTokenizer(line, " ");
				while(++j < N){
					long travels = Long.valueOf(tok.nextToken()).longValue();
					routes[i][j] = travels;
				}
			}
			
			for(Object o : tempEdges){
				Edge e = (Edge) o;
				e.cost = e.cost * routes[e.from-1][e.to-1];
//				System.out.println(e.index + " is " + e.cost);
				allEdges.add(e);// Update priority queue
			}
			buff.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private void performKruskal() {
		int size = allEdges.size();
		for (int i = 0; i < size; i++) {
			Edge curEdge = (Edge) allEdges.first();
			if (allEdges.remove(curEdge)) {
				// successful removal from priority queue: allEdges

				if (nodesAreInDifferentSets(curEdge.from, curEdge.to)) {
					// System.out.println("Nodes are in different sets ...");
					HashSet src, dst;
					int dstHashSetIndex;

					if (nodes[curEdge.from].size() > nodes[curEdge.to].size()) {
						// have to transfer all nodes including curEdge.to
						src = nodes[curEdge.to];
						dst = nodes[dstHashSetIndex = curEdge.from];
					} else {
						// have to transfer all nodes including curEdge.from
						src = nodes[curEdge.from];
						dst = nodes[dstHashSetIndex = curEdge.to];
					}

					Object srcArray[] = src.toArray();
					int transferSize = srcArray.length;
					for (int j = 0; j < transferSize; j++) {
						// move each node from set: src into set: dst
						// and update appropriate index in array: nodes
						if (src.remove(srcArray[j])) {
							dst.add(srcArray[j]);
							nodes[((Integer) srcArray[j]).intValue()] = nodes[dstHashSetIndex];
						} else {
							// This is a serious problem
//							System.out.println("Something wrong: set union");
//							System.exit(1);
						}
					}

					allNewEdges.add(curEdge);
					// add new edge to MST edge vector
				} else {
					// System.out.println("Nodes are in the same set ... nothing to do here");
				}

			} else {
				// This is a serious problem
//				System.out
//						.println("TreeSet should have contained this element!!");
//				System.exit(1);
			}
		}
	}

	private boolean nodesAreInDifferentSets(int a, int b) {
		// returns true if graph nodes (a,b) are in different
		// connected components, ie the set for 'a' is different
		// from that for 'b'
		return (!nodes[a].equals(nodes[b]));
	}

	private void printFinalEdges() {
//		System.out.println("The minimal spanning tree generated by "
//				+ "\nKruskal's algorithm is: ");
		long total = 0;
		while (!allNewEdges.isEmpty()) {
			// for each edge in Vector of MST edges
			Edge e = (Edge) allNewEdges.firstElement();
			System.out.println(e.index);
			total += e.cost;
			allNewEdges.remove(e);
		}
		System.out.flush();
		System.out.close();
//		System.out.println("Total: " + total);
	}

	class Edge implements Comparator {
		// Inner class for representing edge+end-points
		public int from, to, index;
		public long cost;

		public Edge() {
			// Default constructor for TreeSet creation
		}

		public Edge(int f, int t, long c, int k) {
			// Inner class constructor
			from = f;
			to = t;
			cost = c;
			index = k;
		}

		public int compare(Object o1, Object o2) {
			// Used for comparisions during add/remove operations
			long cost1 = ((Edge) o1).cost;
			long cost2 = ((Edge) o2).cost;
			int from1 = ((Edge) o1).from;
			int from2 = ((Edge) o2).from;
			int to1 = ((Edge) o1).to;
			int to2 = ((Edge) o2).to;

			if (cost1 < cost2)
				return (-1);
			else if (cost1 == cost2 && from1 == from2 && to1 == to2)
				return (0);
			else if (cost1 == cost2)
				return (-1);
			else if (cost1 > cost2)
				return (1);
			else
				return (0);
		}

		public boolean equals(Object obj) {
			// Used for comparisions during add/remove operations
			Edge e = (Edge) obj;
			return (cost == e.cost && from == e.from && to == e.to);
		}
	}

}